<?php
namespace Teknisa\SM\Controller;

use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Teknisa\SM\Service\Sm00004108RenovacaoContratoService;
use Teknisa\SM\Controller\BirtReport;
use Teknisa\Libs\Util\SMTP as SMTPService;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\DTO\Row;

class Sm00004108RenovacaoContratoController {
    
    protected $renovacaoservice;
    protected $birtController;
    protected $smtpservice;
    protected $environment;
    
    public function __construct(Sm00004108RenovacaoContratoService $renovacaoservice, SMTPService $smtpservice, BirtReport $birtController, Environment $environment) {
        $this->renovacaoservice = $renovacaoservice;
        $this->smtpservice      = $smtpservice;
        $this->birtController   = $birtController;
        $this->environment      = $environment;
    }
    
    /**
     * Faz a validação das conexões e renova todos os contratos que permitem renovação automática e 
     * venceram na data anteriror a data atual
    **/
    public function renovarContratosAutomaticamente() {
        $organizacoes = $this->renovacaoservice->getOrganizacoesSM();
        
        foreach($organizacoes as $organizacao) {
            try {
                $this->renovacaoservice->deletarLogErroRenovacao($organizacao["NRORG_ID"]);
                $paramsGeral = $this->renovacaoservice->getParametrosGeracaoRelatorio($organizacao["NRORG_ID"]);
                
                $contratosrenovados = $this->renovacaoservice->renovarContratosAutomaticamente($organizacao["NRORG_ID"]);
            }
            catch(\Exception $e) {
                echo "ORGANIZAÇÃO: " . $organizacao["NRORG_ID"] . " | ERRO ao renovar contratos automaticamente: " . $e->getMessage();
            }
                
            
            if ($paramsGeral && $paramsGeral["DSEMAILALERTRENOVCONT"]) {
                try {
                    if (count($contratosrenovados) > 0)
                        $this->enviarRelatorioContratosRenovados($paramsGeral, $organizacao["NRORG_ID"]);
                }
                catch(\Exception $e) {
                    echo "ORGANIZAÇÃO: " . $organizacao["NRORG_ID"] . " | ERRO ao gerar relatório de contratos renovados: " . $e->getMessage();
                }
                
                
                if ($paramsGeral["IDPERIODALERTRENOVCONT"] != "N") {
                    try {
                        $qtdContratosSemRenov = $this->renovacaoservice->getContratosSemRenovacaoAutomatica($organizacao["NRORG_ID"], $paramsGeral["IDPERIODALERTRENOVCONT"]);
                        if ($qtdContratosSemRenov > 0) {
                            $dataAtual = date_create();
                            switch ($paramsGeral["IDPERIODALERTRENOVCONT"]) {
                                case 'D': {
                                    $this->enviarRelatorioContratosVencSemRenovacao($paramsGeral, $organizacao["NRORG_ID"]);
                                    break;
                                }
                                case 'S': {
                                    if ($dataAtual->format("w") == '0')
                                        $this->enviarRelatorioContratosVencSemRenovacao($paramsGeral, $organizacao["NRORG_ID"]);
                                    break;
                                }
                                case 'M': {
                                    if ($dataAtual->format("d") == '01')
                                        $this->enviarRelatorioContratosVencSemRenovacao($paramsGeral, $organizacao["NRORG_ID"]);
                                    break;
                                }
                            }
                        }
                    }
                    catch(\Exception $e) {
                        echo "ORGANIZAÇÃO: " . $organizacao["NRORG_ID"] . " | ERRO ao gerar relatório de contratos a vencer: " . $e->getMessage();
                    }
                }
            }
        }
    }
    
    /**
     * Efetua a renovação de um contrato quanto solicitado
    **/
    public function renovarContrato(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $nrseqcontrato = array(
                array(
                    "NRSEQCONTRATO"   => $row["NRSEQCONTRATO"],
                    "NRTIPOORCAMENTO" => $row["NRTIPOORCAMENTO"]
                )
            );
            $datainivigent = $row["DTINIVIGENCONT"];
            $nrdiasProrrog = $row["NRDIASPRORGCONT"];
            $idreajustar   = $row["IDREAJUSTEAUTOMATICO"];
            
            $this->renovacaoservice->renovarContrato($nrseqcontrato, $nrorg, false, $datainivigent, $nrdiasProrrog, $idreajustar, $cdoperador);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * envia o relatorio contratos_renovados_automaticamente
    **/
    public function enviarRelatorioContratosRenovados($dadosEmail, $nrorg) {
        $remetente = "sac@teknisa.com";
        $assunto   = "Relatório de Renovação Automática de Contratos do SM"; 
        
        $destinatario  = $dadosEmail["DSEMAILALERTRENOVCONT"];
        $destinatario  = str_replace(", ", " ", $destinatario);
        $destinatario  = str_replace(",", " ", $destinatario);
        $destinatarios = explode(" ", $destinatario);
        
        if(count($destinatarios) > 0){
            $paramsTemplate = array(
                "SUBJECT"                 => '',
                "URL_LOGO_CLIENTE"        => '',
                "STRING_SUPERIOR_DIREITA" => '',
                "TITLE_EMAIL"             => 'Renovação Automática de Contrato',
                "FOOTER_TEMPLATE"         => '<img src="https://www.teknisa.com/emails/img/rodape-sm.png" width="590" height="84" />',
                "TEXT_EMAIL"              => '<p>Você está recebendo, anexo a este e-mail, o relatório do processo de renovação automática de contratos do sistema SM.</p>
                                              <p>&nbsp;</p>
                                              <p>O intuito desta lista é informar os novos valores e períodos de vigência dos contratos vencidos até a data de ontem e as falhas no processo, permitindo a identificação do motivo da não renovação de determinados contratos.</p>
                                              <p>&nbsp;</p>
                                              <p>Para esses contratos que apresentaram falhas, é necessário executar as devidas correções e realizar a renovação manual pela tela \'Renovação de Contratos\'.</p>'
            );
            
            $corpo = $this->getTemplateEmailTeknisa($paramsTemplate);
            $anexo = $this->geraRelatorioContratosRenovados($nrorg);

            $tamanho = filesize($anexo);
            $tamanho = ($tamanho/1024)/1024;
            
            if ($tamanho <= 15) {
                $this->smtpservice->addAttachment($anexo);
            } else {
                unlink($anexo);
                throw new \Exception("Não foi possível enviar o E-mail. Não é permitido anexar documentos acima de 15Mb.");
            }
            
            $this->smtpservice->setFrom($remetente);
            $this->smtpservice->setFromName($remetente);
            $this->smtpservice->setSubject($assunto);
            
            $validaEnvioEmail = $this->smtpservice->sendMail($corpo, $destinatarios);
            
            if (is_file($anexo))
                unlink($anexo);
            
            return $validaEnvioEmail;
        }
        
        return false;
    }
    
    /**
     * envia o relatorio contratos_vencidos_sem_renovacao
    **/
    public function enviarRelatorioContratosVencSemRenovacao($dadosEmail, $nrorg) {
        $remetente = "sac@teknisa.com";
        $assunto   = "Relatório de Contratos Vencidos sem Renovação Automática"; 
        
        $destinatario  = $dadosEmail["DSEMAILALERTRENOVCONT"];
        $destinatario  = str_replace(", ", " ", $destinatario);
        $destinatario  = str_replace(",", " ", $destinatario);
        $destinatarios = explode(" ", $destinatario);
        
        if(count($destinatarios) > 0) {
            $periodo = array(
                "D" => "já venceram ou vencem hoje",
                "S" => "já venceram ou vencem em até 7 dias",
                "M" => "já venceram ou vencem até o fim deste mês"
            );
            
            $paramsTemplate = array(
                "SUBJECT"                 => '',
                "URL_LOGO_CLIENTE"        => '',
                "STRING_SUPERIOR_DIREITA" => '',
                "TITLE_EMAIL"             => 'Contratos a Vencer',
                "FOOTER_TEMPLATE"         => '<img src="https://www.teknisa.com/emails/img/rodape-sm.png" width="590" height="84" />',
                "TEXT_EMAIL"              => '<p>Você está recebendo, anexo a este e-mail, o relatório dos contratos que '. $periodo[$dadosEmail["IDPERIODALERTRENOVCONT"]] .' e não possuem renovação automática.</p>
                                              <p>&nbsp;</p>
                                              <p>Caso haja necessidade de reiteração, deve-se executá-la manualmente pela tela de \'Renovação de Contratos\'.</p>'
            );
            
            $corpo = $this->getTemplateEmailTeknisa($paramsTemplate);
            $anexo = $this->geraRelatorioContratosVencidos($dadosEmail["IDPERIODALERTRENOVCONT"], $nrorg);

            $tamanho = filesize($anexo);
            $tamanho = ($tamanho/1024)/1024;
            
            if ($tamanho <= 15) {
                $this->smtpservice->addAttachment($anexo);
            } else {
                unlink($anexo);
                throw new \Exception("Não foi possível enviar o E-mail. Não é permitido anexar documentos acima de 15Mb.");
            }
            
            $this->smtpservice->setFrom($remetente);
            $this->smtpservice->setFromName($remetente);
            $this->smtpservice->setSubject($assunto);
            
            $validaEnvioEmail = $this->smtpservice->sendMail($corpo, $destinatarios);
            
            if (is_file($anexo))
                unlink($anexo);
            
            return $validaEnvioEmail;
        }
        
        return false;
    }
    
    /**
     * Gera o template teknisa com a mensagem do email
    **/
    public function getTemplateEmailTeknisa(array $dadosSubstituiTemplate){
        $template = new \Teknisa\SM\Util\Mail\Templates\Teknisa\TemplateTeknisa();
        return $template->setValoresEmail($dadosSubstituiTemplate);
    }
    
    /**
     * Gera o anexo do relatório de recursos que devem ser desalocados
    **/
    public function geraRelatorioContratosRenovados($nrorg) {
        $rowBirt = new Row();
        $rowBirt->set("__report_name", "contratos_renovados_automaticamente");
        $rowBirt->set("NRORG", $nrorg);
        
        $requestBirt = new Request\Row($rowBirt, "POST", "/geraBirtReport", null);
        $response    = new Response();
        $this->birtController->createRemoteReport($requestBirt, $response);
        
        if (isset($response->getMessages()[0])) {
            $urlRelatorio = $response->getMessages()[0]->getMessage();
            return $this->salvarRelatorioRecursos($urlRelatorio, "contratos_renovados_automaticamente");
        }
    }
    
    /**
     * Gera o anexo do relatório de recursos que devem ser desalocados
    **/
    public function geraRelatorioContratosVencidos($periodo, $nrorg) {
        $rowBirt = new Row();
        $rowBirt->set("__report_name", "contratos_vencidos_sem_renovacao_auto");
        $rowBirt->set("PERIODO", $periodo);
        $rowBirt->set("NRORG", $nrorg);
        
        $requestBirt = new Request\Row($rowBirt, "POST", "/geraBirtReport", null);
        $response    = new Response();
        $this->birtController->createRemoteReport($requestBirt, $response);
        
        if (isset($response->getMessages()[0])) {
            $urlRelatorio = $response->getMessages()[0]->getMessage();
            return $this->salvarRelatorioRecursos($urlRelatorio, "contratos_vencidos_sem_renovacao_auto");
        }
    }
    
    /**
     * Salva o relatorio de recursos 
    **/
    public function salvarRelatorioRecursos($urlRelatorio, $nameRelatorio){
        $microtime   = microtime(true);
        $microtime   = $microtime = str_replace(".", "_", $microtime);
        $nomeArquivo = $microtime . "_" . $nameRelatorio;
        
        $report = @file_get_contents($urlRelatorio);
        if ($report === false) {
            throw new \Exception("Servidor de emissão de relatórios indisponível. Contate o administrador de rede.");
        }
    
        $reportPath = sys_get_temp_dir() . DIRECTORY_SEPARATOR . "$nomeArquivo.pdf";
        file_put_contents($reportPath, $report);
        
        unset($report);
        return $reportPath;
    }
    
    /**
     * Altera os parâmetros "IDRENOVACAOAUTOMATICA" e "IDREAJUSTEAUTOMATICO" para os contratos selecionados
    **/
    public function salvarRenovReajAuto(Request\Row $request, Response $response) {
        try {
            $row        = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
            
            $nrseqcontrato         = $row["NRSEQCONTRATO"];
            $idrenovacaoautomatica = $row["IDRENOVACAOAUTOMATICA"];
            $idreajusteautomatico  = $row["IDREAJUSTEAUTOMATICO"];
            
            $this->renovacaoservice->salvarRenovReajAuto($nrseqcontrato, $idrenovacaoautomatica, $idreajusteautomatico, $nrorg, $cdoperador);
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    /**
     * Renova e reajusta os contratos selecionados
    **/
    public function renovContrMassa(Request\Row $request, Response $response) {
        try {
            $contratos  = $request->getRow();
            $nrorg      = $this->environment->getNrOrg();
            $cdoperador = $this->environment->getCdOperador();
    
            $valida = $this->renovacaoservice->renovContrMassa($contratos, $nrorg, $cdoperador);
            
            $response->addDataSet(new DataSet("retorno", array("valida" => $valida)));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    /**
     * Recebe os dados de Indice Financeiro
    **/
    public function getDadosIndFinanc(Request\Row $request, Response $response) {
        try {
            $row  = $request->getRow();
            
            $nrseqcontrato = $row["NRSEQCONTRATO"];
            $nrorg         = $this->environment->getNrOrg();
             
            $dados = $this->renovacaoservice->getDadosIndFinanc($nrseqcontrato, $nrorg);
            
            $response->addDataSet(new DataSet("dados", $dados));
        } catch (\Exception $e) {
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
}



?>