<?php

namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;

class ApropriacaoRecursoFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    /**
     * Insere um registro de apropiação de recurso
     */
    
    public function inserirApropriacaoRecurso(array $params){
        
        $sql = "INSERT INTO FCT_APROPAGENDAMENTO (
                    NRAPROPAGENDAMENTO,     
                    NRRECURSOSAGENDCONT,         
                    NRPARCNEGRECURSO,  
                	IDSTATUSAPROPRIACAO,
                	DTAPROPRIACAO,
                    QTHORASAPROPRIADAS,
                	NRPARCINTERLOCUTOR,
                	NRORG,
                	IDATIVO,
                	DTINCLUSAO,  
                	NRORGINCLUSAO,
                	CDOPERINCLUSAO,
                	DTULTATU,
                	NRORGULTATU, 
                	CDOPERULTATU,
                	IDDIASEMANA,
                	IDDIAUTIL
                )
                VALUES (
                    :NRAPROPAGENDAMENTO,     
                    :NRRECURSOSAGENDCONT,         
                    :NRPARCNEGRECURSO,  
                	:IDSTATUSAPROPRIACAO,
                	:DTAPROPRIACAO,
                    :QTHORASAPROPRIADAS,
                	:NRPARCINTERLOCUTOR,
                	:NRORG,
                	:IDATIVO,
                	 SYSDATE,  
                	:NRORGINCLUSAO,
                	:CDOPERINCLUSAO,
                	:DTULTATU,
                	:NRORGULTATU, 
                	:CDOPERULTATU,
                	:IDDIASEMANA,
                	:IDDIAUTIL
                )";
        
        $std = $this->connection->prepare($sql);

        $std->execute($params);
    }
    
    /**
     * Deleta um registro de apropiação de recurso
     */
    public function deletarApropriacaoRecurso(array $params, $diaSemana) {
        $sql = "DELETE FROM FCT_APROPAGENDAMENTO 
                 WHERE NRRECURSOSAGENDCONT    = :NRRECURSOSAGENDCONT
                   AND TRUNC(DTAPROPRIACAO) >= :DTAPROPRIACAO
                   AND NRORG                 = :NRORG
                   AND IDDIASEMANA NOT IN(".$diaSemana.")";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Deleta um registro de apropiação de recurso
     */
    public function deletarDiaApropriacaoRecurso(array $params) {
        $sql = "DELETE FROM FCT_APROPAGENDAMENTO 
                 WHERE NRRECURSOSAGENDCONT    = :NRRECURSOSAGENDCONT
                   AND TRUNC(DTAPROPRIACAO)   = :DTAPROPRIACAO
                   AND NRORG                  = :NRORG";
        
        $std = $this->connection->prepare($sql);
        $std->execute($params);
    }
    
    /**
     * Faz consuta  de validação de datas no banco 
     */
    
    public function recuperarParaValidacao(array $params){
        $dados = "SELECT NRAPROPAGENDAMENTO,
                         TO_CHAR(DTAPROPRIACAO, 'DD/MM/YYYY') AS DTAPROPRIACAO
                    FROM FCT_APROPAGENDAMENTO
                   WHERE NRPARCNEGRECURSO = :NRPARCNEGRECURSO
                     AND NRORG            = :NRORG
                   ORDER BY DTAPROPRIACAO";
        
        $consulta = $this->connection->prepare($dados);// consulta recebe o SQL deṕois de preparado
        $consulta->execute($params);// Cosulta ja com o sql preprado execulta com os paramentros
        return $consulta->fetchAll();// neste momento o metodo fechAll retorna um array de acordo com a consulta para o metodo chamador service.
    }
    
    /**
     * pesquisa apropiação com recurso e retona a quantidade 
     */
    
    public function recuperarApropriacaoComApontamento(array $params){
        $dados = "SELECT COUNT(APR.NRAPROPAGENDAMENTO) AS QUANTIDADE
                    FROM FCT_APROPAGENDAMENTO APR
                   INNER JOIN FCT_APONTAMENTOAPROPAGEN APO
                      ON APR.NRAPROPAGENDAMENTO = APO.NRAPROPAGENDAMENTO
                     AND APR.NRORG              = APO.NRORG
                   WHERE APR.NRRECURSOSAGENDCONT   = :NRRECURSOSAGENDCONT
                     AND APR.NRORG                 = :NRORG
                     AND TRUNC(APR.DTAPROPRIACAO) >= :DTAPROPRIACAO";
        
        $consulta = $this->connection->prepare($dados);// consulta recebe o SQL deṕois de preparado
        $consulta->execute($params);// Cosulta ja com o sql preprado execulta com os paramentros
        return $consulta->fetchAll();// neste momento o metodo fechAll retorna um array de acordo com a consulta para o metodo chamador service.
    }
    
    public function getApropriacoesRecursoPorPeriodo(array $params) {
        $sql = "SELECT NRAPROPAGENDAMENTO
                  FROM FCT_APROPAGENDAMENTO
                 WHERE NRRECURSOSAGENDCONT = :NRRECURSOSAGENDCONT
                   AND NRORG               = :NRORG
                   AND DTAPROPRIACAO BETWEEN :DTINI AND :DTFIN";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
        return $ps->fetchAll();
    }
    
    public function inserirApontamento(array $params) {
        $sql = "INSERT INTO FCT_APONTAMENTOAPROPAGEN (
                    NRAPONTAMENTOAPROPAGEN , NRAPROPAGENDAMENTO , IDTIPOAPONTAMENTO ,
                    IDMOTIVOSAIDA          , DTHRINICIOAPONT    , DTHRFIMCIOAPONT   ,
                    QTTEMPOAPONT           , DSOBSAPONTAMENTO   , NRORG             ,
                    IDATIVO                , DTINCLUSAO         , NRORGINCLUSAO     ,
                    CDOPERINCLUSAO
                )
                VALUES (
                    :NRAPONTAMENTOAPROPAGEN , :NRAPROPAGENDAMENTO , :IDTIPOAPONTAMENTO ,
                    :IDMOTIVOSAIDA          , :DTHRINICIOAPONT    , :DTHRFIMCIOAPONT   ,
                    :QTTEMPOAPONT           , :DSOBSAPONTAMENTO   , :NRORG             ,
                    :IDATIVO                , SYSDATE             , :NRORGINCLUSAO     ,
                    :CDOPERINCLUSAO
                )";
        
        $ps = $this->connection->prepare($sql);
        $ps->execute($params);
    }
    
    public function updateApropriacaoRecurso(array $params){
        
    }
}

