<?php

namespace Teknisa\SM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\SM\Util\Utilities;

class FctContratoGridFactory {
    
    protected $connection;
    
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
    }
    
    public function find($conditions, $limit, $offset, $orderBy) {
        $filtro = Utilities::createQueryFiltered($conditions);
        $stringOrderBy = Utilities::getStringOrderBy($orderBy);
        
        $sql = "SELECT * FROM (
                    SELECT CONT.NRSEQCONTRATO,
                           CONT.NRCONTRATOFCT,
                           CONT.NRVERSAOCONTRATO,
                           CONT.IDSITUACAOCONTRATO,
                           CONT.CDFILIALORCA,
                           FILIAL.NMFILIAL,
                           (SELECT SUM(VRTOTITEMCOMDESC)
                              FROM FCT_NEGOCIACAOCONTRATO
                             WHERE NRSEQCONTRATO = CONT.NRSEQCONTRATO
                               AND NRORG = CONT.NRORG) AS VRTOTALCONT,
                           NVL(CONT.DTINIVIGENCONT, CONT.DTINICIOCONTRATO) AS DTINIVIGENCONT,
                           NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO) AS DTFINVIGENCONT,
                           TO_CHAR(NVL(CONT.DTINIVIGENCONT, CONT.DTINICIOCONTRATO), 'DD/MM/YYYY') AS DTINICONTRATO,
                           TO_CHAR(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO), 'DD/MM/YYYY') AS DTFINCONTRATO,
                           CONT.DTINICIOCONTRATO,
                           CONT.DTFIMCONTRATO,
                           CONT.NRPARCNEGOCIO,
                           PARC.NMPRINCIPALPARC,
                           PARC.NMSECUNDARIPARC,
                           PARC.NRINSCRICAOPARC,
                           CONT.DSREFERENCIACONT,
                           CONT.CDOPERRESPONSAVEL,
                           OPERRESP.NMOPERADOR AS NMOPERRESP,
                           CONT.NRSEQMODCONTRA,
                           MODCONT.NMMODCONTRATO,
                           CONT.NRSEQCONTRATOPAI,
                           CONT.CDCENTCUST,
                           CENTCUST.NMCENTCUST,
                           CONT.NRORG,
                           CONT.IDATIVO,
                           CONT.DTINCLUSAO,
                           CONT.CDOPERINCLUSAO,
                           CONT.NRORGINCLUSAO,
                           CONT.DTULTATU,
                           CONT.CDOPERULTATU,
                           CONT.NRORGULTATU,
                           NVL(REFE.IDGERADOPOR, 'CONTRATO') AS IDGERADOPOR,
                           REFE.NRSEQORCAMENTO,
                           DECODE(REFE.IDGERADOPOR, 'PROPOSTA', ORC.NRORCAMENTO, NULL) AS NRORCAMENTO,
                           ORC.NRORCAMENTOMODELO,
                           NVL(CONT.NRTIPOORCAMENTO, REFE.NRTIPOORCAMENTO) AS NRTIPOORCAMENTO,
                           ESCOPO.NMTIPOORCAMENTO,
                           NVL(ESCOPO.IDUTILIZAVARPRECO, 'N') AS IDUTILIZAVARPRECO,
                           CONT.IDREAJUSTEAUTOMATICO,
                           CONT.IDRENOVACAOAUTOMATICA,
                           CONT.IDASSELETRDIGITAL,
                           CONT.NRDIASPRAZOCONT,
                           NVL(CONT.NRDIASPRORGCONT, CONT.NRDIASPRAZOCONT) AS NRDIASPRORGCONT,
                           CONT.NRCONTRATOFCT || ' - ' || CONT.DSREFERENCIACONT AS NMCONTRATO,
                           CONT.NRTIPORESCISAO,
                           CONT.VRPENDENCIAFIN,
                           CONT.CDOPERSOLSUSPENSAO,
                           CONT.DSJUSTIFCASUSPENSAO,
                           OPERSUSP.NMOPERADOR AS OPERSOLSUSPENSAO
                      FROM FCT_CONTRATO CONT
                      LEFT JOIN PARCNEGOCIO PARC ON CONT.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                       AND CONT.NRORG = PARC.NRORG
                      LEFT JOIN OPERADOR OPERRESP ON CONT.CDOPERRESPONSAVEL = OPERRESP.CDOPERADOR
                       AND CONT.NRORG = OPERRESP.NRORG
                      LEFT JOIN FILIAL ON CONT.CDFILIALORCA = FILIAL.CDFILIAL
                       AND CONT.NRORG = FILIAL.NRORG
                      LEFT JOIN CENTCUST ON CONT.CDCENTCUST = CENTCUST.CDCENTCUST
                       AND CONT.NRORG = CENTCUST.NRORG
                      LEFT JOIN FCT_MODELOCONTRATO MODCONT ON CONT.NRSEQMODCONTRA = MODCONT.NRSEQMODCONTRA
                       AND CONT.NRORG = MODCONT.NRORG
                      LEFT JOIN FCT_REFEORCACONT REFE ON CONT.NRSEQCONTRATO = REFE.NRSEQCONTRATO
                       AND CONT.NRORG = REFE.NRORG
                      LEFT JOIN FCT_ORCAMENTO ORC ON REFE.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                       AND REFE.NRORG = ORC.NRORG
                      LEFT JOIN FCT_TIPOORCAMENTO ESCOPO ON NVL(CONT.NRTIPOORCAMENTO, REFE.NRTIPOORCAMENTO) = ESCOPO.NRTIPOORCAMENTO
                       AND CONT.NRORG = ESCOPO.NRORG
                      LEFT JOIN OPERADOR OPERSUSP ON CONT.CDOPERSOLSUSPENSAO = OPERSUSP.CDOPERADOR
                     ORDER BY DTINCLUSAO DESC
                )
                WHERE 1=1 $filtro $stringOrderBy";
        
        $sql = Utilities::getPaginatedQuery($sql, $limit, $offset);
        
        $ps = $this->connection->prepare($sql);
        $ps->execute();
        $dataset = $ps->fetchAll();
        return $dataset;
    }
    
    public function findHierarquia($conditions, $cdoperador, $limit, $offset, $orderBy) {
        $filtro = Utilities::createQueryFiltered($conditions);
        $stringOrderBy = Utilities::getStringOrderBy($orderBy);
        
        $sql = "SELECT * FROM (
                    SELECT CONT.NRSEQCONTRATO,
                    CONT.NRCONTRATOFCT,
                    CONT.NRVERSAOCONTRATO,
                    CONT.IDSITUACAOCONTRATO,
                    CONT.CDFILIALORCA,
                    FILIAL.NMFILIAL,
                    (SELECT SUM(VRTOTITEMCOMDESC)
                    FROM FCT_NEGOCIACAOCONTRATO
                    WHERE NRSEQCONTRATO = CONT.NRSEQCONTRATO
                    AND NRORG = CONT.NRORG) AS VRTOTALCONT,
                    NVL(CONT.DTINIVIGENCONT, CONT.DTINICIOCONTRATO) AS DTINIVIGENCONT,
                    NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO) AS DTFINVIGENCONT,
                    TO_CHAR(NVL(CONT.DTINIVIGENCONT, CONT.DTINICIOCONTRATO), 'DD/MM/YYYY') AS DTINICONTRATO,
                    TO_CHAR(NVL(CONT.DTFINVIGENCONT, CONT.DTFIMCONTRATO), 'DD/MM/YYYY') AS DTFINCONTRATO,
                    CONT.DTINICIOCONTRATO,
                    CONT.DTFIMCONTRATO,
                    CONT.NRPARCNEGOCIO,
                    PARC.NMPRINCIPALPARC,
                    PARC.NMSECUNDARIPARC,
                    PARC.NRINSCRICAOPARC,
                    CONT.DSREFERENCIACONT,
                    CONT.CDOPERRESPONSAVEL,
                    OPERRESP.NMOPERADOR AS NMOPERRESP,
                    CONT.NRSEQMODCONTRA,
                    MODCONT.NMMODCONTRATO,
                    CONT.NRSEQCONTRATOPAI,
                    CONT.CDCENTCUST,
                    CENTCUST.NMCENTCUST,
                    CONT.NRORG,
                    CONT.IDATIVO,
                    CONT.DTINCLUSAO,
                    CONT.CDOPERINCLUSAO,
                    CONT.NRORGINCLUSAO,
                    CONT.DTULTATU,
                    CONT.CDOPERULTATU,
                    CONT.NRORGULTATU,
                    NVL(REFE.IDGERADOPOR, 'CONTRATO') AS IDGERADOPOR,
                    REFE.NRSEQORCAMENTO,
                    DECODE(REFE.IDGERADOPOR, 'PROPOSTA', ORC.NRORCAMENTO, NULL) AS NRORCAMENTO,
                    ORC.NRORCAMENTOMODELO,
                    NVL(CONT.NRTIPOORCAMENTO, REFE.NRTIPOORCAMENTO) AS NRTIPOORCAMENTO,
                    ESCOPO.NMTIPOORCAMENTO,
                    NVL(ESCOPO.IDUTILIZAVARPRECO, 'N') AS IDUTILIZAVARPRECO,
                    CONT.IDREAJUSTEAUTOMATICO,
                    CONT.IDRENOVACAOAUTOMATICA,
                    CONT.IDASSELETRDIGITAL,
                    CONT.NRDIASPRAZOCONT,
                    NVL(CONT.NRDIASPRORGCONT, CONT.NRDIASPRAZOCONT) AS NRDIASPRORGCONT,
                    CONT.NRCONTRATOFCT || ' - ' || CONT.DSREFERENCIACONT AS NMCONTRATO,
                    CONT.NRTIPORESCISAO,
                    CONT.VRPENDENCIAFIN
               FROM FCT_CONTRATO CONT
               LEFT JOIN PARCNEGOCIO PARC ON CONT.NRPARCNEGOCIO = PARC.NRPARCNEGOCIO
                AND CONT.NRORG = PARC.NRORG
               LEFT JOIN OPERADOR OPERRESP ON CONT.CDOPERRESPONSAVEL = OPERRESP.CDOPERADOR
                AND CONT.NRORG = OPERRESP.NRORG
               LEFT JOIN FILIAL ON CONT.CDFILIALORCA = FILIAL.CDFILIAL
                AND CONT.NRORG = FILIAL.NRORG
               LEFT JOIN CENTCUST ON CONT.CDCENTCUST = CENTCUST.CDCENTCUST
                AND CONT.NRORG = CENTCUST.NRORG
               LEFT JOIN FCT_MODELOCONTRATO MODCONT ON CONT.NRSEQMODCONTRA = MODCONT.NRSEQMODCONTRA
                AND CONT.NRORG = MODCONT.NRORG
               LEFT JOIN FCT_REFEORCACONT REFE ON CONT.NRSEQCONTRATO = REFE.NRSEQCONTRATO
                AND CONT.NRORG = REFE.NRORG
               LEFT JOIN FCT_ORCAMENTO ORC ON REFE.NRSEQORCAMENTO = ORC.NRSEQORCAMENTO
                AND REFE.NRORG = ORC.NRORG
               LEFT JOIN FCT_TIPOORCAMENTO ESCOPO ON NVL(CONT.NRTIPOORCAMENTO, REFE.NRTIPOORCAMENTO) = ESCOPO.NRTIPOORCAMENTO
                AND CONT.NRORG = ESCOPO.NRORG
               LEFT JOIN (
                    SELECT *
                      FROM CRM_HIERARQUIAOPER
                     WHERE CDOPERADOR = :CDOPERADORLOGADO) HIERARPAI
                 ON CONT.NRORG = HIERARPAI.NRORG
              WHERE (((CONT.CDOPERRESPONSAVEL IN
                      (SELECT :CDOPERADORLOGADO
                         FROM DUAL
                        UNION ALL SELECT CDOPERADOR
                         FROM CRM_HIERARQUIAOPER
                        WHERE (CRM_HIERARQUIAOPER.CDFILIAL = HIERARPAI.CDFILIAL
                               OR CRM_HIERARQUIAOPER.CDFILIAL IS NULL) CONNECT BY
                          PRIOR CRM_HIERARQUIAOPER.NRHIERARQUIAOPER = CRM_HIERARQUIAOPER.NRHIERARQUIAPAI
                          START WITH CRM_HIERARQUIAOPER.CDOPERADOR = HIERARPAI.CDOPERADOR))
                    OR CONT.CDOPERRESPONSAVEL IS NULL
                    OR CONT.CDOPERRESPONSAVEL = :CDOPERADORLOGADO)
                   OR HIERARPAI.IDOPERMASTER = 'S')
                ORDER BY DTINCLUSAO DESC
                )
                WHERE 1=1 $filtro $stringOrderBy";
        
        $sql = Utilities::getPaginatedQuery($sql, $limit, $offset);
        
        $ps = $this->connection->prepare($sql);
        $ps->execute(array("CDOPERADORLOGADO" => $cdoperador));
        $dataset = $ps->fetchAll();
        return $dataset;
    }
    
}